#ifndef APPLICATION_H_
#define APPLICATION_H_

#include "Platform.h"
#include <Core/Base/TGlobalPtrBase.h>
#include <Core/Log/Log.h>

/**
 * Application class
 */
class Application: public redBlocks::Core::TGlobalPtrBase<Application>
{
  public:
    /**
     * Constructor. Creates and initializes the application modules and associates them with each other.
     */
    Application() :
      mFlashTimer( this ),
      mCommandEvent( this ),
      mLedOn( true ),
      mFlashing( true )
    {
      // set initial Led state
      Platform::Led::setOutput( mLedOn );

      // start the timer for periodic callbacks
      mFlashTimer.startPeriodic( Platform::OS::MilliSec<500>::value );

      // callbacks are disabled by default and need to be enabled manually
      Platform::TerminalComInterface::enableRecvDataCallback();
    }

    /**
     * The application's mainloop: Sleeps until events are pending and processes them.
     */
    void run()
    {
      while ( true )
      {
        Platform::OS::processEvents();
        Platform::AwakeLed::setOutput( false );
        Platform::enterSleepMode();
        Platform::AwakeLed::setOutput( true );
      }
    }

    /**
     * Must be called each time, when a byte is received via the terminal
     * communication interface
     */
    void onCommandData()
    {
      while ( 0 != Platform::TerminalComInterface::getRecvData( &mCommandByte, 1 ) )
      {
        mCommandEvent.schedule();
      }
    }

  private:
    /** Convenience definition for our class */
    typedef Application ThisType;

    /**
     * This method is called each time mFlashTimer fires: It toggles the Led.
     */
    static void isrTimerCallback( ThisType* t )
    {
      t->mLedOn = !t->mLedOn;
      Platform::Led::setOutput( t->mLedOn );
    }

    /**
     * Event handler callback that is invoked when mCommandEvent is fired.
     */
    static void commandEventCallback( ThisType* t, u8 callCnt )
    {
      switch ( t->mCommandByte )
      {
        case '0':
        case '1':
          if ( t->mFlashing )
          {
            t->mFlashing = false;
            t->mFlashTimer.stop();
          }
          t->mLedOn = '1' == t->mCommandByte;
          Platform::Led::setOutput( t->mLedOn );
          RB_LOG_DEBUG( "Switching LED " << ( t->mLedOn ? "on" : "off" ) );
          break;
        case 'f':
          if ( !t->mFlashing )
          {
            t->mFlashing = true;
            t->mFlashTimer.startPeriodic( Platform::OS::MilliSec<500>::value );
          }
          RB_LOG_DEBUG( "Starting to flash LED" );
          break;
        default:
          RB_LOG_DEBUG( "Received invalid command" );
          break;
      }
    }

    /**
     * the timer that is used to flash the LED: we use an interrupt based timer,
     * in order to get constant flash intervals
     */
    Platform::OS::TIsrTimer<ThisType, &isrTimerCallback> mFlashTimer;

    /**
     * the event that is scheduled, when a new command byte has been received
     */
    Platform::OS::TEvent<ThisType, &commandEventCallback> mCommandEvent;

    /**
     * the current led state
     */
    bool mLedOn;
    /**
     * true while the LED is flashing
     */
    bool mFlashing;
    /**
     * the last received command
     */
    u8 mCommandByte;

};

#endif // APPLICATION_H_
